from setuptools import setup, find_packages

setup(
    name = "lubdub",
    version = "0.0.1.4",
    description = "ECG/EKG, RR, NN, HRV, and heartrate calculation toolkit",
    url="https://bitbucket.org/nthmost/lubdub",
    author = "Naomi Most",
    author_email = "naomi@corewellness.io",
    license = "Apache 2.0",
    zip_safe = False,
    packages = find_packages(),
    )

